<%@ Page Language="C#" Debug="false" Trace="false" EnableViewState="false" %>
<%@ import Namespace="System" %>
<%@ import Namespace="System.IO" %>
<%@ import Namespace="System.Xml" %>
<%@ import Namespace="System.Data" %>
<%@ import Namespace="System.Text" %>
<%@ import Namespace="System.Drawing" %>
<%@ import Namespace="System.Drawing.Imaging" %>
<script runat="server">

    // ################ Settings ################
    const string COMIC_DIRECTORY = @"Comics\";
    const string COMICRENDERS_DIRECTORY = @"ComicRenders\";
    const string FILE_PANELLAYOUTS = @"PanelLayouts.xml";

    const string COMIC_RENDER_DATE_FORMAT = @"_yyyyMMdd_hhmmss";

    // Where does a textbox tail appear in relation to the textbox?
    private enum TailPosition : byte
    {
        Unknown, Top, Bottom, Left, Right
    }

    const int TAIL_BASEWIDTH_PRECENT = 30; // Percentage of the textbox side length - Width of tail base
        
    void Page_Load (Object sender, EventArgs e)
    {
        int iComicID = Convert.ToInt32(Request.QueryString["comicid"]);
        string sComicFilename = Server.MapPath(COMIC_DIRECTORY) + iComicID.ToString() + ".xml";
        
        // Does the comic exist?
        if (File.Exists(sComicFilename))
        {
            Trace.Write(sComicFilename + " exists, loading comic XML");
            
            using (FileStream fsComic = new FileStream(sComicFilename, FileMode.Open, FileAccess.Read, FileShare.ReadWrite))
            {
                XmlDocument oXmlDoc = new XmlDocument();
                oXmlDoc.Load(fsComic);

                XmlNode oRoot = oXmlDoc.DocumentElement;
                
                // Comic data
                int iLayoutID = Convert.ToInt32(oRoot.Attributes["PanelLayoutID"].Value);
                string sName = oRoot.Attributes["Name"].Value;
                DateTime dtReleaseDate = Convert.ToDateTime(oRoot.Attributes["DatePublish"].Value);
                DateTime dtModifiedDate = Convert.ToDateTime(oRoot.Attributes["DateModified"].Value);
                
                Trace.Write("Name: " + sName);
                Trace.Write("Release date: " + dtReleaseDate.ToString() );
                Trace.Write("dtReleaseDate: " + iLayoutID.ToString());

                string sRenderFilename = iComicID.ToString() + dtModifiedDate.ToString(COMIC_RENDER_DATE_FORMAT) + ".png";

                // Is the comic already cached? - If so, redirect straight to it
                if ( File.Exists( Server.MapPath( COMICRENDERS_DIRECTORY + sRenderFilename ) ) & Request.QueryString["preview"]== null )
                    Response.Redirect("/admin/stripeditor/ComicRenders/" + sRenderFilename, true );
                
                                
                // Load panel layout
                XmlNode oPanelLayout = LoadPanelLayout(iLayoutID);
                if (oPanelLayout != null)
                {
                    // Create comic layout
                    using (Bitmap bmpComic = new Bitmap( Convert.ToInt32(oPanelLayout.Attributes["width"].Value), 
                                                         Convert.ToInt32(oPanelLayout.Attributes["height"].Value),
                                                         PixelFormat.Format32bppArgb ))
                    {
                        // Create an array of bitmaps and graphics for each panel
                        Bitmap[] bmpComicPanels = new Bitmap[oPanelLayout.ChildNodes.Count];
                        Graphics[] gfxComicPanels = new Graphics[oPanelLayout.ChildNodes.Count];
                        
                        for (int iPanelID = 1; iPanelID <= oPanelLayout.ChildNodes.Count; iPanelID++)
                        {
                            XmlElement oPanel = (XmlElement)oPanelLayout.ChildNodes[iPanelID - 1]; // Layout settings 
                            
                            Trace.Write(string.Format("Creating panel {0}, width:{1} height:{2}", iPanelID, oPanel.Attributes["width"].Value, oPanel.Attributes["height"].Value));                            
                            
                            // Create panel bitmap
                            bmpComicPanels[iPanelID - 1] = new Bitmap(Convert.ToInt32(oPanel.Attributes["width"].Value),
                                                                     Convert.ToInt32(oPanel.Attributes["height"].Value),
                                                                     PixelFormat.Format32bppArgb);
                            
                            // Create panel graphic
                            gfxComicPanels[iPanelID - 1] = Graphics.FromImage(bmpComicPanels[iPanelID - 1]);

                            // Render settings
                            gfxComicPanels[iPanelID - 1].CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.AssumeLinear;
                            gfxComicPanels[iPanelID - 1].InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
                            gfxComicPanels[iPanelID - 1].TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;
                        }
                        
                        // Prepear background
                        Graphics gfxComic = Graphics.FromImage( bmpComic );
                        gfxComic.Clear(Color.White);

                        // Draw panels background
                        DrawPanelsBackground(ref gfxComicPanels, oRoot, oPanelLayout);                         

                        // Draw sprites and textboxes
                        DrawObjects(ref gfxComicPanels, oRoot, oPanelLayout);

                        // Draw panels border
                        DrawPanelsBorder(ref gfxComicPanels, oRoot, oPanelLayout);                            
                        
                        // Draw seperate panels onto master comic
                        for (int iPanelID = 1; iPanelID <= oPanelLayout.ChildNodes.Count; iPanelID++)
                        {
                            XmlElement oPanel = (XmlElement)oPanelLayout.ChildNodes[iPanelID - 1]; // Layout settings 
                            
                            Rectangle rctPanel = new Rectangle( Convert.ToInt32(oPanel.Attributes["x"].Value),
                                                                Convert.ToInt32(oPanel.Attributes["y"].Value),
                                                                Convert.ToInt32(oPanel.Attributes["width"].Value),
                                                                Convert.ToInt32(oPanel.Attributes["height"].Value) );

                            gfxComic.DrawImageUnscaledAndClipped(bmpComicPanels[iPanelID - 1], rctPanel);
                        }

                        if (Request.QueryString["preview"] != null)
                        {
							CheckUserIsValid();
						
							// Stream comic to browser
							Response.Clear();
							Response.ContentType = "image/png";
							bmpComic.Save(Response.OutputStream, ImageFormat.Jpeg);
							Response.Flush();
							Response.End();
                        }
                        else
                        {
                            // Check comic has reached release date before rendering to disk
                            TimeSpan tsSpan = DateTime.Now.Subtract(dtReleaseDate);

                            if (tsSpan.TotalMinutes >= 0)
                            { // Past release date
                                
                                // Save comic to disk
                                bmpComic.Save(Server.MapPath(COMICRENDERS_DIRECTORY + sRenderFilename ), ImageFormat.Png);
                                Response.Redirect("/admin/stripeditor/ComicRenders/" + sRenderFilename);
                            }
                            else
                            { // Not ready to release

                                Response.Redirect("/admin/stripeditor/images/NotYetReleased.jpg");
                            }
                                
                        }
                        
                    }
                    
                } // END - oPanelLayout != null
            } // END - using fsComic
        }
        else
        {
            Trace.Write(sComicFilename + " does not exist");
        }        
    }
	
    // Check user is valid
    void CheckUserIsValid()
    {
		// Security is enabled/disabled via the Web.Config
		if( Convert.ToBoolean( ConfigurationSettings.AppSettings["UserSecurityEnabled"] ) )
	        if ( Session["StrifeStrife_UserID"] == null )
	            Response.Redirect( "../Login.aspx" );
    }	

    // Draw objects - textboxes and sprites
    void DrawObjects(ref Graphics[] gfxComicPanels, XmlNode oRoot, XmlNode oPanelLayout)
    {
        // Loop through each panel and draw in the objects
        for (int iPanelID = 1; iPanelID <= oRoot.ChildNodes.Count; iPanelID++)
        {
            Graphics gfxComic = gfxComicPanels[iPanelID - 1];

            Trace.Write("DrawObjects", "PanelID " + iPanelID.ToString());
            XmlElement oPanel = (XmlElement)oRoot.ChildNodes[iPanelID - 1];

            // Work out the objects draw order based on the Z index
            int[] iZindex = new int[oPanel.ChildNodes.Count];
            int[] iObjectsZSorted = new int[oPanel.ChildNodes.Count];

            // Fill Z index array
            for (int j = 0; j < oPanel.ChildNodes.Count; j++)
                iZindex[j] = Convert.ToInt32(oRoot.ChildNodes[iPanelID - 1].ChildNodes[j].Attributes["zIndex"].Value);

            // Fill object array
            for (int j = 0; j < oPanel.ChildNodes.Count; j++)
                iObjectsZSorted[j] = j;

            // Sort the array based upon Z index
            Array.Sort(iZindex, iObjectsZSorted);

            // Loop through each object
            for (int i = 0; i < oPanel.ChildNodes.Count; i++)
            {
                int iObjectID = iObjectsZSorted[i];
                
                switch (oPanel.ChildNodes[iObjectID].Name)
                {
                    case "Textbox":
                        DrawObject_Textbox(ref gfxComic, oRoot, iPanelID, iObjectID);
                        break;
                    case "Sprite":
                        DrawObject_Sprite(ref gfxComic, oRoot, iPanelID, iObjectID);
                        break;
                }                

            } // END - Object loop
        } // END - Panel loop
    }      

    // Draw textboxes
    void DrawObject_Textbox(ref Graphics gfxComic, XmlNode oRoot, int iPanelID, int iObjectID)
    {
        XmlElement oTextbox = (XmlElement)oRoot.ChildNodes[iPanelID - 1].ChildNodes[iObjectID];

        Rectangle rctTextbox = new Rectangle(Convert.ToInt32(oTextbox.Attributes["x"].Value),
                                             Convert.ToInt32(oTextbox.Attributes["y"].Value),
                                             Convert.ToInt32(oTextbox.Attributes["width"].Value),
                                             Convert.ToInt32(oTextbox.Attributes["height"].Value));

        Rectangle rctText = new Rectangle(rctTextbox.X + 5,
                                          rctTextbox.Y + 5,
                                          rctTextbox.Width - 5,
                                          rctTextbox.Height - 5);

        // Background
        int iAlpha = Convert.ToInt32(oTextbox.Attributes["alpha"].Value);
        if (oTextbox.Attributes["backgroundColour"].Value.ToString().ToLower() == "transparent")
            iAlpha = 0;  
        
        Brush oBrush = new SolidBrush(Color.FromArgb(ConvertRange100to255(iAlpha),
                                                     Color.FromName(oTextbox.Attributes["backgroundColour"].Value)));

        gfxComic.FillRectangle(oBrush, rctTextbox);

        // Background border
        iAlpha = Convert.ToInt32(oTextbox.Attributes["alpha"].Value);
        if (oTextbox.Attributes["borderColour"].Value.ToString().ToLower() == "transparent")
            iAlpha = 0;          

        Pen oPen = new Pen(new SolidBrush(Color.FromArgb(ConvertRange100to255(iAlpha),
                                                         Color.FromName(oTextbox.Attributes["borderColour"].Value))));
        
        
        oPen.Width = Convert.ToInt32(oTextbox.Attributes["borderWidth"].Value);
        if (oPen.Width == 0)
            oPen.Color = Color.Transparent;

        switch (oTextbox.Attributes["borderStyle"].Value.ToLower())
        {
            case "solid":
                oPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Solid;
                break;
            case "dashed":
                oPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;
                break;
            case "dotted":
                oPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
                break;
        }

	// Textbox draw mode
	if ( Convert.ToBoolean( oTextbox.Attributes["tailEnabled"].Value ) )
	        Textbox_DrawTail(ref gfxComic, ref oTextbox, ref oPen, ref oBrush, ref rctTextbox);
	else
		gfxComic.DrawRectangle(oPen, rctTextbox);        // Draw textbox tail (speech bubble)

        // Text
        FontStyle oFontStyle = new FontStyle();
        if (oTextbox.GetAttribute("fontWeight").Length > 0)
            oFontStyle = FontStyle.Bold;

        if (oTextbox.GetAttribute("fontStyle").Length > 0)
            oFontStyle = FontStyle.Italic;

        if (oTextbox.GetAttribute("fontWeight").Length > 0 && oTextbox.GetAttribute("fontStyle").Length > 0)
            oFontStyle = FontStyle.Bold | FontStyle.Italic;

        Font oFont = new Font(oTextbox.GetAttribute("fontName"), Convert.ToInt32(oTextbox.GetAttribute("fontSize")), oFontStyle);

        StringFormat oStringFormat = new StringFormat();

        // Alignment
        switch (oTextbox.GetAttribute("textAlign"))
        {
            case "left":
                oStringFormat.Alignment = StringAlignment.Near;
                break;
            case "center":
                oStringFormat.Alignment = StringAlignment.Center;
                break;
            case "right":
                oStringFormat.Alignment = StringAlignment.Far;
                break;
        }

        gfxComic.DrawString(oTextbox.GetAttribute("text"),
                             oFont,
                             new SolidBrush(Color.FromArgb(ConvertRange100to255(Convert.ToInt32(oTextbox.Attributes["alpha"].Value)),
                                                           Color.FromName(oTextbox.Attributes["colour"].Value))),
                             rctText,
                             oStringFormat);

    }
    
    // Draw sprites
    void DrawObject_Sprite(ref Graphics gfxComic, XmlNode oRoot, int iPanelID, int iObjectID)
    {
        XmlElement oSprite = (XmlElement)oRoot.ChildNodes[iPanelID - 1].ChildNodes[iObjectID];

        // Load sprite
        using (Bitmap bmpSprite = new Bitmap(Server.MapPath(oSprite.Attributes["Image"].Value)))
        {
            Rectangle rctSprite = new Rectangle(Convert.ToInt32(oSprite.Attributes["x"].Value),
                                                Convert.ToInt32(oSprite.Attributes["y"].Value),
                                                Convert.ToInt32(oSprite.Attributes["width"].Value),
                                                Convert.ToInt32(oSprite.Attributes["height"].Value));

            // Flip
            switch (oSprite.Attributes["flip"].Value)
            {
                case "xy":
                    bmpSprite.RotateFlip(RotateFlipType.RotateNoneFlipXY);
                    break;
                case "x":
                    bmpSprite.RotateFlip(RotateFlipType.RotateNoneFlipX);
                    break;
                case "y":
                    bmpSprite.RotateFlip(RotateFlipType.RotateNoneFlipY);
                    break;
            }

            // Set-up sprite alpha 
            ColorMatrix cm = new ColorMatrix();
            cm.Matrix00 = cm.Matrix11 = cm.Matrix22 = cm.Matrix44 = 1;
            cm.Matrix33 = Convert.ToSingle(oSprite.Attributes["alpha"].Value) / 100;

            // Create a new image attribute and set the color matrix to it
            ImageAttributes oImageAttributes = new ImageAttributes();
            oImageAttributes.SetColorMatrix(cm);

            // Draw sprite into panel
            gfxComic.DrawImage(bmpSprite, rctSprite, 0, 0, bmpSprite.Width, bmpSprite.Height, GraphicsUnit.Pixel, oImageAttributes);
        }        
    }

    // Draw panels
    void DrawPanelsBackground(ref Graphics[] gfxComicPanels, XmlNode oRoot, XmlNode oPanelLayout)
    {
        // Loop through each panel and draw it on
        for (int iPanelID = 1; iPanelID <= oPanelLayout.ChildNodes.Count; iPanelID++)
        {
            Graphics gfxComic = gfxComicPanels[iPanelID - 1];

            XmlElement oPanel = (XmlElement)oPanelLayout.ChildNodes[iPanelID - 1]; // Layout settings 
            XmlElement oPanelComic = (XmlElement)oRoot.ChildNodes[iPanelID - 1]; // Comic settings            
 
            Rectangle rctPanel = new Rectangle(0,
                                               0,
                                               Convert.ToInt32(oPanel.Attributes["width"].Value),
                                               Convert.ToInt32(oPanel.Attributes["height"].Value));

            // Draw panel background
            if (oPanelComic.Attributes["BackgroundImage"].Value.Length > 0)
                using (Bitmap bmpBackground = new Bitmap(Server.MapPath(oPanelComic.Attributes["BackgroundImage"].Value)))
                    gfxComic.DrawImage(bmpBackground, rctPanel);
                    // gfxComic.DrawImageUnscaledAndClipped(bmpBackground, rctPanel);
        }
    }

    // Draw panels
    void DrawPanelsBorder(ref Graphics[] gfxComicPanels, XmlNode oRoot, XmlNode oPanelLayout)
    {
        // Loop through each panel and draw it on
        for (int iPanelID = 1; iPanelID <= oPanelLayout.ChildNodes.Count; iPanelID++)
        {
            Graphics gfxComic = gfxComicPanels[iPanelID - 1];

            Trace.Write("DrawPanels", "PanelID " + iPanelID.ToString());
            XmlElement oPanel = (XmlElement)oPanelLayout.ChildNodes[iPanelID - 1]; // Layout settings 
            XmlElement oPanelComic = (XmlElement)oRoot.ChildNodes[iPanelID - 1]; // Comic settings

            // Obtain panel settings
            Pen oPanelBorder = new Pen(Color.FromName(oPanel.Attributes["border-color"].Value));
            oPanelBorder.Width = Convert.ToInt32(oPanel.Attributes["border-width"].Value);

            Rectangle rctPanel = new Rectangle(0,
                                               0,
                                               Convert.ToInt32(oPanel.Attributes["width"].Value),
                                               Convert.ToInt32(oPanel.Attributes["height"].Value));

            Trace.Write("DrawPanels", string.Format("Panel: {0}, {1}, {2}, {3}", rctPanel.X, rctPanel.Y, rctPanel.Width, rctPanel.Height));

            // Draw panel outline
            gfxComic.DrawRectangle(oPanelBorder, rctPanel);
        }
    }    

    // Draw textbox tail (speech bubble)
    void Textbox_DrawTail( ref Graphics gfxComic, ref XmlElement oTextbox, ref Pen oPen, ref Brush oBrush, ref Rectangle rctTextbox )
    {
        if ( Convert.ToBoolean( oTextbox.Attributes["tailEnabled"].Value ) )
        {
            // Where does the tail fall, in relation to the textbox?
            int iTailX = Convert.ToInt32(oTextbox.Attributes["tailX"].Value);
            int iTailY = Convert.ToInt32(oTextbox.Attributes["tailY"].Value);
            
            TailPosition oTailPosition = TailPosition.Unknown;

            if (iTailY < rctTextbox.Y)
                oTailPosition = TailPosition.Top;
            else if (iTailY > (rctTextbox.Y + rctTextbox.Height))
                oTailPosition = TailPosition.Bottom;
            else if (iTailY > rctTextbox.Y && iTailY < (rctTextbox.Y + rctTextbox.Height))
            {
                if (iTailX < rctTextbox.X)
                    oTailPosition = TailPosition.Left;
                else if (iTailX > (rctTextbox.X + rctTextbox.Width))
                    oTailPosition = TailPosition.Right;
            }

            if (oTailPosition != TailPosition.Unknown)
            {
                Trace.Write("Textbox_DrawTail", "oTailPosition: " + oTailPosition.ToString());

                Point[] oPoints; // Text dimensions
                int iTailBaseWidth = 0; // Dynamically size the text width
                int iTailBorderComp = Convert.ToInt32(Math.Floor(oPen.Width/2)); // Compensate for the border width
                
                switch (oTailPosition)
                {   
                    case TailPosition.Top:
                        iTailBaseWidth = Convert.ToInt32((Convert.ToDouble(rctTextbox.Width) / 100) * TAIL_BASEWIDTH_PRECENT);
                        
                        oPoints = new Point[]
                        {
                            new Point( rctTextbox.Left, rctTextbox.Top ), // Top left        
                            new Point( rctTextbox.Left, rctTextbox.Bottom ), // Bottom left                              
                            new Point( rctTextbox.Right, rctTextbox.Bottom ), // Bottom right
                            new Point( rctTextbox.Right, rctTextbox.Top ), // Top right
                            new Point( rctTextbox.Left + (rctTextbox.Width/2) + (iTailBaseWidth/2), rctTextbox.Top + iTailBorderComp ), // Textbox end point                            
                            new Point( iTailX, iTailY ), // Tail tip
                            new Point( rctTextbox.Left + (rctTextbox.Width/2) - (iTailBaseWidth/2), rctTextbox.Top + iTailBorderComp ),  // Textbox start point                            
                            new Point( rctTextbox.Left, rctTextbox.Top ) // Back to top left                                                          
                        };                          
                        break;
                    case TailPosition.Bottom:
                        iTailBaseWidth = Convert.ToInt32((Convert.ToDouble(rctTextbox.Width) / 100) * TAIL_BASEWIDTH_PRECENT);
                        
                        oPoints = new Point[]
                        {
                            new Point( rctTextbox.Left, rctTextbox.Top ), // Top left        
                            new Point( rctTextbox.Left, rctTextbox.Bottom ), // Bottom left                        
                            new Point( rctTextbox.Left + (rctTextbox.Width/2) - (iTailBaseWidth/2), rctTextbox.Bottom - iTailBorderComp),  // Textbox start point
                            new Point( iTailX, iTailY ), // Tail tip
                            new Point( rctTextbox.Left + (rctTextbox.Width/2) + (iTailBaseWidth/2), rctTextbox.Bottom - iTailBorderComp), // Textbox end point
                            new Point( rctTextbox.Right, rctTextbox.Bottom ), // Bottom right
                            new Point( rctTextbox.Right, rctTextbox.Top ), // Top right
                            new Point( rctTextbox.Left, rctTextbox.Top ) // Back to top left                        
                        };                        
                        break;
                    case TailPosition.Left:
                        iTailBaseWidth = Convert.ToInt32((Convert.ToDouble(rctTextbox.Height) / 100) * TAIL_BASEWIDTH_PRECENT);
                        
                        oPoints = new Point[]
                        {
                            new Point( rctTextbox.Left, rctTextbox.Top ), // Top left
                            new Point( rctTextbox.Left + iTailBorderComp, rctTextbox.Top + (rctTextbox.Height/2) - (iTailBaseWidth/2) ),  // Textbox start point
                            new Point( iTailX, iTailY ), // Tail tip
                            new Point( rctTextbox.Left + iTailBorderComp, rctTextbox.Top + (rctTextbox.Height/2) + (iTailBaseWidth/2) ), // Textbox end point
                            new Point( rctTextbox.Left, rctTextbox.Bottom ), // Bottom left
                            new Point( rctTextbox.Right, rctTextbox.Bottom ), // Bottom right
                            new Point( rctTextbox.Right, rctTextbox.Top ), // Top right
                            new Point( rctTextbox.Left, rctTextbox.Top ) // Back to top left
                        };                          
                        break;
                    case TailPosition.Right:
                        iTailBaseWidth = Convert.ToInt32((Convert.ToDouble(rctTextbox.Height) / 100) * TAIL_BASEWIDTH_PRECENT);
                        
                        oPoints = new Point[]
                        {
                            new Point( rctTextbox.Left, rctTextbox.Top ), // Top left
                            new Point( rctTextbox.Left, rctTextbox.Bottom ), // Bottom left
                            new Point( rctTextbox.Right, rctTextbox.Bottom ), // Bottom right                            
                            new Point( rctTextbox.Right - iTailBorderComp, rctTextbox.Top + (rctTextbox.Height/2) + (iTailBaseWidth/2) ), // Textbox end point                            
                            new Point( iTailX, iTailY ), // Tail tip
                            new Point( rctTextbox.Right - iTailBorderComp, rctTextbox.Top + (rctTextbox.Height/2) - (iTailBaseWidth/2) ),  // Textbox start point
                            new Point( rctTextbox.Right, rctTextbox.Top ), // Top right
                            new Point( rctTextbox.Left, rctTextbox.Top ) // Back to top left                            
                        };                        
                        break;                                                
                    default:
                        oPoints = new Point[0];
                        break;
                }

                // Fill tail background
                gfxComic.FillPolygon(oBrush, oPoints);
                
                // Draw outline
                gfxComic.DrawLines(oPen, oPoints);
            }
            else
                Trace.Write("Textbox_DrawTail()", "Tail position is unknown, aborting");
        }    
    }
        
    // Load panel layout information
    XmlNode LoadPanelLayout(int iLayoutID)
    {
        using (FileStream fsPanelLayouts = new FileStream( Server.MapPath(FILE_PANELLAYOUTS), FileMode.Open, FileAccess.Read, FileShare.ReadWrite))
        {
            XmlDocument oXmlDoc = new XmlDocument();
            oXmlDoc.Load(fsPanelLayouts);
            
            // Search for layout ID
            XmlNode oRoot = oXmlDoc.DocumentElement;

            for (int i = 0; i < oRoot.ChildNodes.Count; i++)
            {
                if ( Convert.ToInt32( oRoot.ChildNodes[i].Attributes["id"].Value ) == iLayoutID)
                    return oRoot.ChildNodes[i];

            }

            Trace.Write("LoadPanelLayout", "Layout not found");
            return null; // Not found       
        }
    }
    
    // Convert a 0-100 range number to a 0-255 range
    int ConvertRange100to255(int iInput)
    {
        return Convert.ToInt32(( (double) 255 / 100) * iInput);
    }
    
</script>